@@ -8,4 +8,4 @@ loafit <- corrHLfit(cbind(npos,ntot-npos)~elev1+elev2+elev3+elev4+maxNDVI1+seNDV
           init.corrHLfit=list(Nugget=0.1),ranFix=list(nu=0.5)) 
 
 expect_equal(attr(loafit$corrPars,"type"),list(nu="fix",Nugget="outer",rho="outer"))
-expect_equal(loafit$corrPars$Nugget,0.04577515,tolerance=1e-05)
\ No newline at end of file
+expect_equal(loafit$corrPars$Nugget,0.04578246,tolerance=1e-5) ## NB flat p_bv for Nugget
